<?php
/**
 * Created by JetBrains PhpStorm.
 * User: leonardooliveira
 * Date: 12/05/14
 * Time: 11:36
 * To change this template use File | Settings | File Templates.
 */

namespace Financeiro\Logic\ContasReceber\Cnab\Retorno;

use Financeiro\Entities\Ajbolrec;
use Financeiro\Entities\Boletreceber;
use Financeiro\Entities\GeralRec;
use Financeiro\Logic\ContasReceber\Cnab\Retorno\LeitorRetornoSantander;
use Financeiro\Entities\Errcnabr;
use Geral\Entities\Banco;
use Geral\Util\Ambiente;
abstract class LeitorRetorno {

    const CODIGO_COMPENSACAO_SANTANDER = "033";
    const CODIGO_COMPENSACAO_BRADESCO  = "237";
    const CODIGO_COMPENSACAO_ITAU      = "341";
    const CODIGO_COMPENSACAO_BRASIL    = "001";
    const CODIGO_COMPENSACAO_CAIXA     = "104";

    protected $erros = array();
    protected $ocorrencias = array();
    protected $contlinha = 0;
    protected $cdempresa;
    protected $cdbanco;
    protected $retornos = array();

    public function getErros()
    {
        return $this->erros;
    }

    public function getRetornos()
    {
        return $this->retornos;
    }

    public static function factory($cdempresa, $cdbanco, $arquiuvo)
    {
        try {
            if ($cdbanco === self::CODIGO_COMPENSACAO_SANTANDER)
                return new LeitorRetornoSantander($arquiuvo, $cdempresa, $cdbanco);
            elseif ($cdbanco === self::CODIGO_COMPENSACAO_BRADESCO)
                return new LeitorRetornoBradesco($arquiuvo, $cdempresa, $cdbanco);
            elseif($cdbanco === self::CODIGO_COMPENSACAO_ITAU)
                return new LeitorRetornoItau($arquiuvo, $cdempresa, $cdbanco);
            elseif($cdbanco === self::CODIGO_COMPENSACAO_BRASIL)
                return new LeitorRetornoBrasil($arquiuvo, $cdempresa, $cdbanco);
            elseif($cdbanco === self::CODIGO_COMPENSACAO_CAIXA)
                return new LeitorRetornoCaixa($arquiuvo, $cdempresa, $cdbanco);
            else throw new \Exception('O Códido de Compensação informado não pertece a nenhum banco reconhecido.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public abstract function leArquivo();

    protected function gravaErros($cdlinha, $erro, $iderroGeral = false){
        $this->erros[] = array(
            'CDLINHA' => str_pad($cdlinha, 4, '0', STR_PAD_LEFT),
            'DSERRO'  => $erro,
            'IDGERAL' => $iderroGeral
        );
    }

    public function preparaOcorrencias($cdbanco){
        /** @var $retorno Retorno */
        foreach($this->retornos as $retorno){

            $ocorrencia = $retorno->getOcorrencia();
            $tamOcorrencia = strlen($retorno->getOcorrencia());

            for($i = 0; $i<$tamOcorrencia; $i+=2){
                $cderro = substr($ocorrencia, $i, 2);
                if($cderro !== "  " && $cderro != '06' && $cderro != '17' && $cderro != '09'){

                    /** @var $errcnabr Errcnabr */
                    $errcnabr = Errcnabr::findOneByCdbancoAndCderrcnabrAndNrorg($cdbanco, $cderro, Ambiente::getNrorg());

                    $dserro = $cderro;
                    if($errcnabr)
                        $dserro.= " - {$errcnabr->getDserrcnabr()}";

                    $this->erros[] = array(
                        'CDLINHA' => $retorno->getCdlinha(),
                        'CDERRO'  => $cderro,
                        'DSERRO'  => $dserro,
                        'IDGERAL' => false
                    );
                }
            }
        }
    }

    protected function getTipoBaixaPagamento($cdfilial, $nrlanbolrec){

        /** @var $boletreceber Boletreceber */
        $boletreceber = Boletreceber::findOneByNrlanbolrecAndCdfilial($nrlanbolrec, $cdfilial);

        if($boletreceber and $boletreceber->getCdtpbaixarec()){
            return $boletreceber->getCdtpbaixarec();
        }
    }

    protected function getReducont($cdfilial, $nrlanbolrec){
        /** @var $boletreceber Boletreceber */
        $boletreceber = Boletreceber::findOneByNrlanbolrecAndCdfilial($nrlanbolrec, $cdfilial);

        if($boletreceber and $boletreceber->getCdreducont()){
            return $boletreceber->getCdreducont();
        }
    }

    private function getBanco(){
        $banco = Banco::findOneByNrorgAndCdbanco(Ambiente::getNrorg(), $this->cdbanco);
        return $banco;
    }

    public function validaBoletos(){

        $contasVerificadas = array();
        /** @var $retorno Retorno  */
        foreach($this->retornos as &$retorno){

            //Se for um boleto
            if($retorno->getBoleto()){

                $retorno->setSituacaoOk(true);

                if(!$retorno->getCdfilial() or !$retorno->getNrlanbolrec()){
                    self::gravaErros($retorno->getCdlinha(),'Boleto não cadastrado');
                    $retorno->setSituacaoOk(false);
                }
                else{
                    /** @var $boletreceber Boletreceber */
                    $boletreceber = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($retorno->getNrlanbolrec(),
                        $retorno->getCdfilial(),
                        \Ambiente::getNrorg());
                    if(! $boletreceber){
                        self::gravaErros($retorno->getCdlinha(),'Boleto não cadastrado.');
                        $retorno->setSituacaoOk(false);
                    }
                    else{
                        if($boletreceber->getNrbaixarec()){
                            self::gravaErros($retorno->getCdlinha(),'Boleto encontra-se baixado.');
                            $retorno->setSituacaoOk(false);
                        }
                        if($this->cdempresa != $boletreceber->getCdempresa()){
                            self::gravaErros($retorno->getCdlinha(), 'Empresa do boleto é diferente da empresa fnformada.');
                            $retorno->setSituacaoOk(false);
                        }
                    }
                    $vrpago = round($retorno->getVrpago(), 2);
                    if($boletreceber && $retorno->getIdsitucnabr() == '90'){
                        $vrfinal = round($retorno->getVrnominal() + $retorno->getVracrescimo() - $retorno->getVrdesconto(), 2);
                        if($vrpago != $vrfinal){
                            self::gravaErros($retorno->getCdlinha(), 'Valor final do boleto está diferente do valor cadastrado.');
                            $retorno->setSituacaoOk(false);
                        }
                    }

                    if(round($retorno->getVracrescimo(),2)>0){
                        /** @var $geralpag GeralRec */
                        $geralpag = GeralRec::findOneByNrorg(Ambiente::getNrorg());
                        if(! $geralpag->getCdajacrerec()){
                            self::gravaErros($retorno->getCdlinha(),
                                'Tipo de ajuste não está parametrizado.');
                            $retorno->setSituacaoOk(false);
                        }
                        else{
                            $ajbolpag = Ajbolrec::findOneByCdfilialAndNrlanbolrecAndCdajuctrecAndNrorg(
                                $retorno->getCdfilial(),
                                $retorno->getNrlanbolrec(),
                                $geralpag->getCdajacrerec(),
                                \Ambiente::getNrorg()
                            );

                            if($ajbolpag){
                                self::gravaErros($retorno->getCdlinha(),'Boleto já possui ajuste cadastrado para o
                                                                         tipo de ajuste parametrizado');
                                $retorno->setSituacaoOk(false);
                            }
                        }
                    }

                    if(round($retorno->getVrdesconto(),2)>0){
                        /** @var $geralpag GeralRec */
                        $geralpag = GeralRec::findOneByNrorg(Ambiente::getNrorg());
                        if(! $geralpag->getCdajdescrec()){
                            self::gravaErros($retorno->getCdlinha(),
                                'Tipo de ajuste não está parametrizado .');
                            $retorno->setSituacaoOk(false);
                        }
                        else{
                            $ajbolpag = Ajbolrec::findOneByCdfilialAndNrlanbolrecAndCdajuctrecAndNrorg(
                                $retorno->getCdfilial(),
                                $retorno->getNrlanbolrec(),
                                $geralpag->getCdajdescrec(),
                                \Ambiente::getNrorg()
                            );

                            if($ajbolpag){
                                self::gravaErros($retorno->getCdlinha(),'Boleto já possui ajuste cadastrado para o
                                                                         tipo de ajuste parametrizado');
                                $retorno->setSituacaoOk(false);
                            }
                        }
                    }
                    if($boletreceber){
                        if(! $boletreceber->getCdreducont()){
                            self::gravaErros($retorno->getCdlinha(),'Boleto não possui conta corrente cadastrada.');
                            $retorno->setSituacaoOk(false);
                        }
                    }

                    else{/*
                        if($retorno->getCdagencia() and $retorno->getCdconta()){

                            // Verifica se a conta e agencia do boleto ja foram verificadas
                            $key = array_search(array('CDBANCO' => $this->cdbanco,
                                                      'CDAGENCIA' => $retorno->getCdagencia(),
                                                      'CDCONTCORR' => $retorno->getCdconta(),
                                                      'CDEMPRESA' => $this->cdempresa,
                                                      'CDREDUCONT' => $retorno->getCdreducont()
                                                      )
                                               ,$contasVerificadas);

                            if (!$key)
                            {

                                $banco = self::getBanco();

                                $lenPictCont = strlen($banco->getCdpictcont());

                                if(strpos($banco->getCdpictcont(),'-') !== false)
                                    $lenPictCont = strpos($banco->getCdpictcont(),'-');


                                $repositorio = Contcorr::getRepository();
                                $result = $repositorio->getContCorrRetornoCnab($this->cdempresa,
                                                                               $this->cdbanco,
                                                                               $retorno->getCdagencia(),
                                                                               $retorno->getCdconta(),
                                                                               \Ambiente::getNrorg(),
                                                                               $lenPictCont);
                                if($result){
                                    $result = $result[0];
                                    $contasVerificadas[] = array($result['CDBANCO'],
                                                                 $result['CDAGENCIA'],
                                                                 $result['CDCONTCORR'],
                                                                 $result['CDEMPRESA'],
                                                                 $result['CDREDUCONT']);

                                    $key = count($contasVerificadas)-1;
                                }


                            }

                            if($key == null)
                                $conta = null;
                            else
                                $conta = $contasVerificadas[$key];
                            if(! $conta or $boletpagar->getCdreducont() != $conta['CDREUCONT']){

                                if((! $conta) or ($conta['CDBANCO'] != $boletpagar->getReducont()->getCdbanco() or
                                   $conta['CDEMPRESA'] != $boletpagar->getCdempresa())){

                                    self::gravaErros($retorno->getCdlinha(),
                                                     "Conta corrente de débito {$this->cdbanco} /
                                                     {$retorno->getCdagencia()} / {$retorno->getCdconta()}
                                                     é diferente da conta corrente do boleto.");
                                    $retorno->setSituacaoOk(false);
                                }
                            }
                        }
                        else{
                            if($boletpagar->getReducont()->getCdbanco() != $this->cdbanco or
                               $boletpagar->getCdempresa() != $this->cdempresa){

                                self::gravaErros($retorno->getCdlinha(),
                                                 "Conta corrente de débito {$this->cdbanco} /
                                                 {$retorno->getCdagencia()} / {$retorno->getCdconta()}
                                                 é diferente da conta corrente do boleto.");
                                $retorno->setSituacaoOk(false);
                            }
                        }*/
                    }

                    if ($boletreceber) {
                        if (!$boletreceber->getCdtpbaixarec()) {
                            self::gravaErros(
                                $retorno->getCdlinha(), 'Boleto não possui tipo de baixa a receber cadastrado'
                            );
                            $retorno->setSituacaoOk(false);
                        }
                    }
                }
            }
        }
    }
}