<?php
namespace Teknisa\Model\Gestaopessoas\Repositories;

use HCM\Util\Repositories;

class EsoCadastro extends \Doctrine\ORM\EntityRepository {
    
    public function getEnviosEvento($nrorg, $nmevento, $nrestruturam, $nrtpambiente, $idativo = 'S', $nrchave = null, $nrtabela = null, $idoperacao = array(), $nrenvioexcl=null) {
        try{
            $esoCadastro = Repositories::ESO_CADASTRO;
            $esoEvento = Repositories::ESO_EVENTO;
            
            $db = \Setforms2\Shared\Registry::getEntityManagerConnection();
            $params = array(
                'nrorg' => $nrorg,
                'nmevento' => $nmevento,
                'nrestruturam' => $nrestruturam,
                'nrtpambiente' => $nrtpambiente,
                'idativo' => $idativo,
                'nrchave' => $nrchave,
                'nrtabela' => $nrtabela,
            );
            $sql = "SELECT envio
                      FROM $esoCadastro envio
                     WHERE envio.nrorg = :nrorg
                       AND envio.nmevento = :nmevento
                       AND envio.nrestruturam = :nrestruturam 
                       AND envio.nrtpambiente = :nrtpambiente
                       AND envio.idativo = :idativo
                       AND (envio.nrchave = :nrchave OR :nrchave IS NULL)
                       AND (envio.nrtabela = :nrtabela OR :nrtabela IS NULL)";
            
            if (!empty($idoperacao)) {
                $sql .= " AND (envio.idoperacao IN ('".implode("','",$idoperacao)."'))";
            }
           
            return $this->getEntityManager()->createQuery($sql)->execute($params);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function ativaEvento($nrenvio, $nrorg, $nrestruturam, $cdOper, $dataUltAtul){
        try{
            $esoCadastro = Repositories::ESO_CADASTRO;
            $sql = 
            "UPDATE $esoCadastro esoCad
               SET esoCad.idativo = 'S',
                   esoCad.nrorgultatu = :p_nrorg,
                   esoCad.dtultatu = :p_dtultatu,
                   esoCad.cdoperultatu = :p_cdoperador
             WHERE esoCad.nrorg = :p_nrorg
               AND esoCad.nrestruturam = :p_nrestruturam
               AND esoCad.idativo = 'N'
               AND esoCad.nrenvio = :p_nrenvio";
            $params = array(
                "p_nrorg" => $nrorg,
                "p_nrestruturam" => $nrestruturam,
                "p_nrenvio" => $nrenvio,
                "p_cdoperador" => $cdOper,
                "p_dtultatu" => $dataUltAtul
            );

        return $this->getEntityManager()->createQuery($sql)->execute($params);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function deleteEnviosExclusaoEmpregador($nrorg, $nrestruturam) {
        try{
            $esoCadastro = Repositories::ESO_CADASTRO;
            $esoNaoperiodico = Repositories::ESO_NAOPERIODICO;
            $esoPeriodico = Repositories::ESO_PERIODICO;
            $esoEnviocadastros = Repositories::ESO_ENVIOCADASTROS;
            $esoEvento = Repositories::ESO_EVENTO;
            
            $db = \Setforms2\Shared\Registry::getEntityManagerConnection();
            
            $params = array(
                'nrorg' => $nrorg,
                'nrestruturam' => $nrestruturam,
                'nrtpambiente' => 2,
            );
            $sql = "DELETE FROM $esoCadastro e
                     WHERE e.nrorg = :nrorg
                       AND e.nrestruturam = :nrestruturam 
                       AND e.nrtpambiente = :nrtpambiente";
            $this->getEntityManager()->createQuery($sql)->execute($params);
            
            $params = array(
                'nrorg' => $nrorg,
                'nrestruturam' => $nrestruturam,
                'nrtpambiente' => 2,
            );
            $sql = "DELETE FROM $esoNaoperiodico e
                     WHERE e.nrorg = :nrorg
                       AND e.nrestruturam = :nrestruturam 
                       AND e.nrtpambiente = :nrtpambiente";
            $this->getEntityManager()->createQuery($sql)->execute($params);
            
            $params = array(
                'nrorg' => $nrorg,
                'nrestruturam' => $nrestruturam,
                'nrtpambiente' => 2,
            );
            $sql = "DELETE FROM $esoPeriodico e
                     WHERE e.nrorg = :nrorg
                       AND e.nrestruturam = :nrestruturam 
                       AND e.nrtpambiente = :nrtpambiente";
            $this->getEntityManager()->createQuery($sql)->execute($params);
            
            $params = array(
                'nrorg' => $nrorg,
                'nrestruturam' => $nrestruturam,
                'nrtpambiente' => 2,
            );
            $sql = "DELETE FROM $esoEnviocadastros e
                     WHERE e.nrorg = :nrorg
                       AND e.nrestruturam = :nrestruturam 
                       AND e.nrtpambiente = :nrtpambiente";
            $this->getEntityManager()->createQuery($sql)->execute($params);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function updateEnvios($idStatus, $cdErro, $dsObservacao, $dsObservacao2=null, $dtRecepcao, $nrProtocolo, $allNrEnvio = array(), $nrOrg, $dtUltAtu, $cdOperUltAtu, $nrOrgUltAtu) {
        try{
            $esoCadastro = Repositories::ESO_CADASTRO;
            
            $params = array(
                'idstatus' => $idStatus,
                'cderro' => $cdErro,
                'dsobservacao' => $dsObservacao,
                'dsobservacao2' => $dsObservacao2,
                'dtresposta' => $dtRecepcao,
                'nrprotocolo' => $nrProtocolo,
                'nrorg' => $nrOrg,
                'dtultatu' => $dtUltAtu,
                'cdoperultatu' => $cdOperUltAtu,
                'nrorgultatu' => $nrOrgUltAtu
            );
            $sql = "UPDATE $esoCadastro esocad
                      SET esocad.idstatus = :idstatus,
                      esocad.cderro = :cderro,
                      esocad.dsobservacao = :dsobservacao,
                      esocad.dsobservacao2 = :dsobservacao2,
                      esocad.dtresposta = :dtresposta,
                      esocad.nrprotocolo = :nrprotocolo,
                      esocad.dtultatu = :dtultatu,
                      esocad.cdoperultatu = :cdoperultatu,
                      esocad.nrorgultatu = :nrorgultatu
                     WHERE esocad.nrorg = :nrorg
                       AND (esocad.nrenvio IN ('".implode("','",$allNrEnvio)."'))";
            
            $this->getEntityManager()->createQuery($sql)->execute($params);
        } catch (Exception $e) {
            throw $e;
        }
    }
}